unit adxOLBeforeCheckNames_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, Variants, adxAddIn,
  adxOLBeforeCheckNames_TLB, StdVcl, Classes, Dialogs, Outlook2000;

type
  TADX_OL_BeforeCheckNames = class(TadxAddin, IADX_OL_BeforeCheckNames)
  end;

  TAddInModule = class(TadxCOMAddInModule)
    procedure adxCOMAddInModuleAddInInitialize(Sender: TObject);
    procedure adxCOMAddInModuleAddInFinalize(Sender: TObject);
  private
    FInspectorList: TList;
    FInspectors: TInspectors;
    FMailItem: TMailItem;
  protected
  public
    procedure DoItemSend(ASender: TObject; const Item: IDispatch; var Cancel: WordBool);
    procedure DoNewInspector(ASender: TObject; const Inspector: _Inspector);

    procedure DoInspectorActivate(ASender: TObject);
    procedure DoInspectorClose(ASender: TObject);

    procedure DoBeforeCheckNames(ASender: TObject; var Cancel: WordBool);
  end;

implementation

{$R *.dfm}

procedure TAddInModule.adxCOMAddInModuleAddInInitialize(Sender: TObject);
begin
  FMailItem := nil;
  FInspectors := nil;

  FInspectorList := TList.Create;

  if OutlookApp.ActiveExplorer <> nil then begin
    OutlookApp.OnItemSend := DoItemSend;

    FInspectors := TInspectors.Create(nil);
    FInspectors.ConnectTo(OutlookApp.Inspectors);
    FInspectors.OnNewInspector := DoNewInspector;
  end;
end;

procedure TAddInModule.adxCOMAddInModuleAddInFinalize(Sender: TObject);
begin
  if Assigned(FInspectors) then FreeAndNil(FInspectors);
  if Assigned(FMailItem) then begin
    FMailItem.Disconnect;
    FMailItem := nil;
  end;
  FInspectorList.Free;
end;

procedure TAddInModule.DoNewInspector(ASender: TObject; const Inspector: _Inspector);
var
  IMail: MailItem;
  Insp: TInspector;
begin
  IMail := nil;
  if Assigned(Inspector) and Assigned(Inspector.CurrentItem) then
    Inspector.CurrentItem.QueryInterface(IID__MailItem, IMail);
  if Assigned(IMail) then
    try
      Insp := TInspector.Create(nil);
      Insp.ConnectTo(Inspector);
      Insp.OnActivate := DoInspectorActivate;
      Insp.OnClose := DoInspectorClose;
      FInspectorList.Add(Insp);
    finally
      IMail := nil;
    end;
end;

procedure TAddInModule.DoInspectorActivate(ASender: TObject);
var
  IMail: MailItem;
begin
  IMail := nil;
  if (ASender is TInspector) and Assigned((ASender as TInspector).CurrentItem) then
    (ASender as TInspector).CurrentItem.QueryInterface(IID__MailItem, IMail);
  if Assigned(IMail) then
    try
      if Assigned(FMailItem) then begin
        FMailItem.Disconnect;
        FMailItem := nil;
      end;
      FMailItem := TMailItem.Create(nil);
      FMailItem.ConnectTo(IMail);
      FMailItem.OnBeforeCheckNames := DoBeforeCheckNames;
    finally
      IMail := nil;
    end;
end;

procedure TAddInModule.DoInspectorClose(ASender: TObject);
var
  Index: Integer;
begin
  Index := FInspectorList.IndexOf(ASender);
  if Index >= 0 then begin
    TInspector(FInspectorList[Index]).Disconnect;
    FInspectorList.Delete(Index);
  end;
end;

procedure TAddInModule.DoItemSend(ASender: TObject; const Item: IDispatch;
  var Cancel: WordBool);
begin
  ShowMessage('The OnItemSend event.');
end;

procedure TAddInModule.DoBeforeCheckNames(ASender: TObject; var Cancel: WordBool);
var
  i: Integer;
  To_: string;
  Recipients: string;
begin
  Cancel := False;
  To_ := '';
  Recipients := '';
  try
    if ASender is TMailItem then begin
      To_ := (ASender as TMailItem).To_;
      for i := 1 to (ASender as TMailItem).Recipients.Count do
        Recipients := Recipients +
          (ASender as TMailItem).Recipients.Item(i).Name +
          ' = ' +
          (ASender as TMailItem).Recipients.Item(i).Address +
          #13#10;
    end;
  except
    //
  end;
  ShowMessage('The OnBeforeCheckNames event.' + #13#10 +
    'To: ' + To_ + #13#10 +
    'Recipients:' + #13#10 + Recipients);
end;

initialization
  TadxFactory.Create(ComServer, TADX_OL_BeforeCheckNames, CLASS_ADX_OL_BeforeCheckNames, TAddInModule);

end.
